@@ -33,10 +33,10 @@
 	size_t _bosnode_right_offset       = x == 0 ? offsetof(BOSNode, left_child_node) : offsetof(BOSNode, right_child_node); \
 	size_t _bosnode_left_count_offset  = x == 1 ? offsetof(BOSNode, left_child_count) : offsetof(BOSNode, right_child_count); \
 	size_t _bosnode_right_count_offset = x == 0 ? offsetof(BOSNode, left_child_count) : offsetof(BOSNode, right_child_count)
-#define BOSTREE_LR_LEFT_CHILD(x)  *(BOSNode**)((char*)x + _bosnode_left_offset)
-#define BOSTREE_LR_RIGHT_CHILD(x) *(BOSNode**)((char*)x + _bosnode_right_offset)
-#define BOSTREE_LR_RIGHT_CHILD_COUNT(x) *(unsigned int *)((char*)x + _bosnode_right_count_offset)
-#define BOSTREE_LR_LEFT_CHILD_COUNT(x)  *(unsigned int *)((char*)x + _bosnode_left_count_offset)
+#define BOSTREE_LR_LEFT_CHILD(x)  *(BOSNode**)(void*)((char*)x + _bosnode_left_offset)
+#define BOSTREE_LR_RIGHT_CHILD(x) *(BOSNode**)(void*)((char*)x + _bosnode_right_offset)
+#define BOSTREE_LR_RIGHT_CHILD_COUNT(x) *(unsigned int *)(void*)((char*)x + _bosnode_right_count_offset)
+#define BOSTREE_LR_LEFT_CHILD_COUNT(x)  *(unsigned int *)(void*)((char*)x + _bosnode_left_count_offset)
 
 /* Tree structure */
 struct _BOSTree {
@@ -258,14 +258,11 @@ BOSNode *bostree_insert(BOSTree *tree, void *key, void *data) {
 
 /*
 	Remove a given node from the tree. The argument must be a node, not the
-	associated key! You can look up nodes using bostree_lookup(). The memory
-	for the key/value is not freed, the node structure itself is.
+	associated key! You can look up nodes using bostree_lookup().
 
-	So you must call
-		free(node->key);
-		free(node->value);
-		bostree_remove(tree, node);
-	to cleanly remove an entry.
+	This function decreases the weak reference count of the deleted node by
+	one. Once it reaches zero, the free() helper is called and the node
+	is freed.
 */
 void bostree_remove(BOSTree *tree, BOSNode *node) {
 	if(node == NULL) {
@@ -390,21 +387,29 @@ void bostree_remove(BOSTree *tree, BOSNode *node) {
 	once it has been removed from the tree.
 */
 BOSNode *bostree_node_weak_ref(BOSNode *node) {
+	assert(node->weak_ref_count > 0);
+	assert(node->weak_ref_count < 127);
 	node->weak_ref_count++;
 	return node;
 }
 
 /*
-	Remove the weak reference again, returning NULL if the node was unvalid and
-	the node if it is still valid
+	Remove the weak reference again, returning NULL if the node was invalid and
+	the node if it is still valid.
+
+	Once the weak reference count reaches zero, call the free() helper for
+	the user to delete node->data and node->key. The node structure itself is
+	freed by us.
 */
 BOSNode *bostree_node_weak_unref(BOSTree *tree, BOSNode *node) {
 	BOSNode *retval = node->weak_ref_node_valid ? node : NULL;
+	assert(node->weak_ref_count > 0);
 	if(--node->weak_ref_count == 0) {
 		if(tree->free_function != NULL) {
 			tree->free_function(node);
 		}
 		free(node);
+		return NULL;
 	}
 	return retval;
 }
@@ -525,7 +530,8 @@ unsigned int bostree_rank(BOSNode *node) {
 static void _bostree_print_helper(BOSNode *node, unsigned int indent, unsigned int level) {
 	printf("\033[%d;%dH", level + 1, indent);
 	fsync(0);
-	printf("%s(%d,%d,%d)", (char *)node->key, node->left_child_count, node->right_child_count, node->depth);
+	printf("%s(%d,%d,%d)\n", (char *)node->key, node->left_child_count, node->right_child_count, node->depth);
+	fsync(0);
 
 	if(node->left_child_node != NULL) {
 		_bostree_print_helper(node->left_child_node, indent - (2 << node->depth), level + 1);
